#include <gazebo/gazebo.hh>
#include <gazebo/common/common.hh>
#include <gazebo/physics/physics.hh>
#include <boost/program_options.hpp>
#include <gazebo/transport/transport.hh>
#include <string>

#include "Randomobject.h"
#include "Cube.h"
#include "Cylinder.h"
#include "Sphere.h"

#include <fstream>

using namespace std;
using namespace gazebo;

int main(int _argc, char **_argv) {
	gazebo::setupServer(_argc, _argv);

	gazebo::physics::WorldPtr world = gazebo::loadWorld("factory.world");
	gazebo::transport::PublisherPtr reset_pub;
	gazebo::transport::NodePtr node(new gazebo::transport::Node());
	node->Init();
	reset_pub = node->Advertise<gazebo::msgs::WorldControl>(
			"/gazebo/default/world_control");

	gazebo::msgs::WorldControl w_ctrl;

	transport::PublisherPtr factoryPub = node->Advertise<msgs::Factory>(
			"~/factory");
	msgs::Factory msg;

	std::string modelName = "";

	ofstream dataFile;
	dataFile.open ("dataFile.csv");

	dataFile << "id;type;Mass;material;size_x;size_y;size_z;init_x;init_y;init_z;init_rot_x;init_rot_y;init_rot_z;final_x;final_y;final_z;final_rot_x;final_rot_y;final_rot_z" << endl;

	Random_object *t = NULL;
	srand(time(NULL));
	int k = 0;
	int i = 0;
	while (true) {
		if (i++ > 10 || k == 0) {

			if (k != 0) {

				dataFile << k << ";";
				math::Pose pose =world.get()->GetModel(modelName)->GetWorldPose();


				dataFile << t->getTypeObject() << ";" << t->getMass() << ";" << t->getMaterial() << ";" ;

				dataFile << t->printDataSize();


				dataFile  << t->getPosX()    << ";" << t->getPosY()    << ";" << t->getPosZ()    << ";" ;
				dataFile  << t->getRotPosX() << ";" << t->getRotPosY() << ";" << t->getRotPosZ() << ";" ;



				dataFile  << pose.pos.x << ";" << pose.pos.y << ";" << pose.pos.z << ";" ;
				dataFile <<  pose.rot.GetAsEuler().x << ";" << pose.rot.GetAsEuler().y << ";" << pose.rot.GetAsEuler().z << std::endl;
				world.get()->RemoveModel(modelName);

				delete (t);
			}
			gazebo::common::Time::MSleep(100);
			modelName = "obj" + std::to_string(k++);


			switch (rand() % 3) {
			case 0:
				t = new Cube();
				break;
			case 1:
				t = new Cylinder();
				break;
			case 2:
				t = new Sphere();
				break;
			default:
				t = new Cube();
				break;
			}
			t->setModelName(modelName);
			msg.set_sdf(t->getSdf());

			msgs::Set(msg.mutable_pose(),
					ignition::math::Pose3d(ignition::math::Vector3d(0, 0, 0),
							ignition::math::Quaterniond(0, 0, 0)));

			factoryPub->Publish(msg);
			//end hd
			w_ctrl.mutable_reset()->set_all(true);

			reset_pub->Publish(w_ctrl);
			i = 0;
		}
		gazebo::runWorld(world, 1500);
		gazebo::common::Time::MSleep(100);

	}

	dataFile.close();
	gazebo::shutdown();
}
/*
 #include <gazebo/gazebo.hh>
 #include <gazebo/common/common.hh>
 #include <gazebo/physics/physics.hh>
 #include <boost/program_options.hpp>
 #include <gazebo/transport/transport.hh>


 using namespace gazebo;

 int main(int _argc, char **_argv) {
 gazebo::setupServer(_argc, _argv);
 int i = 0;
 gazebo::physics::WorldPtr world = gazebo::loadWorld("factory.world");

 gazebo::transport::PublisherPtr reset_pub;
 gazebo::transport::NodePtr node(new gazebo::transport::Node());
 node->Init();
 reset_pub = node->Advertise<gazebo::msgs::WorldControl>("/gazebo/default/world_control");

 gazebo::msgs::WorldControl w_ctrl;

 //	transport::PublisherPtr factoryPub = node->Advertise<msgs::Factory>("~/factory");

 //	msgs::Factory msg;



 while (true) {
 if (i++ > 10) {
 std::cout << ".";

 //			msgs::Set(msg.mutable_pose(),
 //					ignition::math::Pose3d(ignition::math::Vector3d(2, -2, 1),
 //							ignition::math::Quaterniond(0, 0, 0)));

 // Send the message
 //			factoryPub->Publish(msg);

 w_ctrl.mutable_reset()->set_all(true);
 reset_pub->Publish(w_ctrl);
 i = 0;

 }
 gazebo::runWorld(world, 1000);
 gazebo::common::Time::MSleep(100);
 }

 gazebo::shutdown();
 }
 */
